import { Factory, ResolutionCondition, UnknownCreator } from '../../types';
import { Token } from '../../pointers';
import { BindingsVault } from '../BindingsVault';
import { ScopeSyntax } from './ScopeSyntax';
export declare class TypeSyntax<T> {
    private readonly vault;
    private readonly token;
    private readonly condition?;
    constructor(vault: BindingsVault, token: Token, condition?: ResolutionCondition | undefined);
    toConstant(value: T): void;
    /**
     * @example <caption>Example usage of factory without arguments.</caption>
     * const someClassFactoryToken = token<Factory<SomeClass>>('Factory<SomeClass>');
     *
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass);
     * // OR
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass, (instance) => instance.init());
     *
     * const someClassFactory = container.get(someClassFactoryToken);
     * const someClassInstance = someClassFactory();
     *
     * console.log(someClassInstance instanceof SomeClass); // -> true
     */
    toFactory(creator: T extends Factory<infer R> ? UnknownCreator<R> : never, initializer?: T extends Factory<infer R> ? (instance: R) => unknown : never): void;
    /**
     * @example <caption>Example usage of factory with arguments.</caption>
     * const someClassFactoryToken = token<Factory<SomeClass, [name: string]>>('Factory<SomeClass>');
     *
     * container
     * .bind(someClassFactoryToken)
     * .toFactory(SomeClass, (instance, name) => instance.setName(name));
     *
     * const someClassFactory = container.get(someClassFactoryToken);
     * const someClassInstance = someClassFactory('Olivia');
     *
     * console.log(someClassInstance instanceof SomeClass); // -> true
     */
    toFactory(creator: T extends Factory<infer R, never[]> ? UnknownCreator<R> : never, initializer: T extends Factory<infer R, infer A> ? (instance: R, ...args: A) => unknown : never): void;
    toInstance<K extends UnknownCreator<T>>(creator: K): ScopeSyntax;
}
